r"""Contain code to check if two objects are equal or not."""

from __future__ import annotations

__all__ = ["objects_are_allclose", "objects_are_equal"]

from coola.equality.interface import objects_are_allclose, objects_are_equal
